
#include "stdafx.h"
#include <fstream>
#include <string>
#include <ctime>
#include "CSV.h"

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


typedef struct {
    POINT upperleft;
    POINT upperright;
    POINT lowerright;
    POINT lowerleft;
} GRIDCOORDS;

typedef struct _dblxy {double fx,fy;} DCOORD;

#define fcoord DCOORD

typedef struct {
  fcoord ulc,urc,lrc,llc;
  int nx,ny;
  int offx, offy, totx, toty;
  fcoord cellsize;
} griddescr;

#define HALF_CELL 0.5f
#define FULL_CELL 1.0f
#define ROUNDUP 0.5


void LoadGridStruct(griddescr *ptrGriddesc,GRIDCOORDS* ptrGridRect,long nCellsX,long nCellsY)
{
	ptrGriddesc->nx = ptrGriddesc->totx = nCellsX;
	ptrGriddesc->ny = ptrGriddesc->toty = nCellsY;
	ptrGriddesc->offx = ptrGriddesc->offy = 0;
	ptrGriddesc->ulc.fx =  ptrGridRect->upperleft.x;
	ptrGriddesc->ulc.fy =  ptrGridRect->upperleft.y;
	ptrGriddesc->urc.fx =  ptrGridRect->upperright.x;
	ptrGriddesc->urc.fy =  ptrGridRect->upperright.y;
	ptrGriddesc->lrc.fx =  ptrGridRect->lowerright.x;
	ptrGriddesc->lrc.fy =  ptrGridRect->lowerright.y;
	ptrGriddesc->llc.fx =  ptrGridRect->lowerleft.x;
	ptrGriddesc->llc.fy =  ptrGridRect->lowerleft.y;
}


BOOL GetFloatCellPosition( float x, float y, int *ulx, int *uly, griddescr *griddesc )
{
	double fx, cfx, fy, cfy; /* Fractions and Complements of positions in X, Y */
	double topx, botx, lefty, righty;

	double gridX;
	double gridY;

	*ulx = *uly = 0; /* default values in case of error */
	
	cfx = (double)x / griddesc->nx; //Denominators checked for zero by ValidImageForCellGen()
	cfy = (double)y / griddesc->ny;
	fx = 1. - cfx;
	fy = 1. - cfy;

	if( fx<0 || cfx<0. || fy<0. || cfy<0. ){
		  return FALSE;
	}

	topx  = fx * griddesc->ulc.fx + cfx * (griddesc->urc.fx+1);
	botx  = fx * griddesc->llc.fx + cfx * (griddesc->lrc.fx+1);
	lefty = fy * griddesc->ulc.fy + cfy * (griddesc->llc.fy+1);
	righty= fy * griddesc->urc.fy + cfy * (griddesc->lrc.fy+1);

	// Compute the coordinates in double precision.
	gridX = fy * topx + cfy * botx;
	gridY = fx * lefty+ cfx * righty;


	*ulx = (int)(gridX+ROUNDUP);
	*uly = (int)(gridY+ROUNDUP);

	return TRUE;
}

BOOL CalcCellCorners(long *ptrGridCoords, long CellX, long CellY, long NumCellCols, long NumCellRows, long *ptrLongCellRect)
{
	GRIDCOORDS * ptrGridRect=(GRIDCOORDS*)ptrGridCoords;
	RECT *ptrCellRect= (RECT*)ptrLongCellRect;
	griddescr griddesc;
	BOOL retVal;

	// FILL IN THE griddesc STRUCT 
	LoadGridStruct(&griddesc,ptrGridRect,NumCellCols,NumCellRows);

	//Calculate the coordinates of the cell
	int cx0=0,cy0=0,cx1=0,cy1=0,pX,pY;
	
	float fCellX=(float) CellX, fCellY=(float) CellY;

	//Top edge
	retVal = GetFloatCellPosition( fCellX+HALF_CELL, fCellY, &pX, &pY, &griddesc);
	if( retVal == FALSE )
	{
		return FALSE;
	}
	cy0=pY;

	//Left edge 
	retVal = GetFloatCellPosition( fCellX, fCellY+HALF_CELL, &pX, &pY, &griddesc);
	if( retVal == FALSE )
	{
		return FALSE;
	}
	cx0=pX;

	//Bottom edge
  	retVal = GetFloatCellPosition( fCellX+HALF_CELL, fCellY+FULL_CELL, &pX, &pY, &griddesc);
	if( retVal == FALSE )
	{
		return FALSE;
	}
	cy1=pY;

	//Right edge
  	retVal = GetFloatCellPosition( fCellX+FULL_CELL, fCellY+HALF_CELL, &pX, &pY, &griddesc);
	if( retVal == FALSE )
	{
		return FALSE;
	}
	cx1=pX;

	//Load the return struct
	ptrCellRect->top = cy0;
	ptrCellRect->left = cx0;
	ptrCellRect->bottom = cy1-1;
	ptrCellRect->right = cx1-1;

	return TRUE;
}

//
// Output the probes intensities to the output file.
//
void OutputPixelData(const char *probeFlag, int probeX, int probeY, const char *genechip, const char *strProbeSetName, CRect &pixelRect, IGDACDATFile *pDatFile ,ofstream &outstr)
{
	CString csvpath;
	CString logpgc;
	CFile File;
	CString gc;
	CString gene;
	int end;
	CString ProbeID;
	
	gc=genechip;
	csvpath=".\\CSV\\"+gc+".csv";
	gene=strProbeSetName;
	if(gene.Find("_3_at")!=-1)
	{
        gene.Replace("_3_at","");
		end=1;
		//i=i+20;
	}else
	{
        gene.Replace("_5_at","");
		end=0;

	}

	if(!File.Open(csvpath, CFile::modeRead|CFile::shareDenyNone)) 	{AfxMessageBox("CSV file not found!"); return;}
	
	CArchive Ar(&File, CArchive::load);
	CString S;
	CString pX;
	CString pY;
	pX.Format("%d", probeX);
	pY.Format("%d", probeY);
	while(Ar.ReadString(S)) { //For all lines of the file
    CCSV CSV;
    CString Field;
    CSV.Set(&Field,S);
	CString CSVGC;
	
	
    while(CSV.GetNextField()) { //for all Fields of the Record
      //The current Field is now stored in CString Field
		if(Field=="AFFX-GapdhMur/M32599_3_at")
		{CSVGC=Field;}
		if(pX==Field)
		{
			CSV.GetNextField();
			if(pY==Field)
			{
                for(int i=0;i<6;i++)
					CSV.GetNextField();

			/*if(CSVGC=="AFFX-GapdhMur/M32599_3_at" && pX=="12" && pY=="1")
			{	
				logpgc="5.807354922";
			}else{
                logpgc=Field;*/
			
			//}
			if(Field!="") logpgc=Field;
		}
		} }}

	for (int ix=pixelRect.left+1; ix<=pixelRect.right-1; ix++)
	{
		for (int iy=pixelRect.top+1; iy<=pixelRect.bottom-1; iy++)
		{
			ProbeID=pX+pY;
			outstr <<
				genechip << "\t" <<
				strProbeSetName << "\t" <<
				gene<<"\t"<<
				ProbeID<<"\t"<<
				end<<"\t"<<
				probeX << "\t" <<
				probeY << "\t" <<
				probeFlag << "\t" <<
				ix+1 << "\t" <<
				iy+1 << "\t";
				
			if(ix+1<1000 && iy+1<1000)
			{outstr << 
			"0"<< ix+1<<"0"<<iy+1<<"\t";}
			else
			{
			if(ix+1<1000)
			{outstr << 
			"0"<< ix+1<<iy+1<<"\t";}
			if(iy+1<1000)
			{outstr << 
			ix+1<<"0"<<iy+1<<"\t";}
			}
			if(ix+1>999 && iy+1>999)
			{outstr << 
			ix+1<<iy+1<<"\t";}
			
			outstr<<pDatFile->GetIntensity(ix, iy) <<"\t\t"<<
					logpgc << endl;
			

		}
	}
}

//
// Given the name of a DAT file, export the pixel data to a text file.
//
CString ExportData(const char *datFile, const char *dataPath, const char *GeneChip,const char *ProbeSet1, const char *ProbeSet2, 
				const char *ProbeSet3,const char *ProbeSet4, const char *ProbeSet5, const char *ProbeSet6,const char *ProbeSet7,
				const char *ProbeSet8, const char *ProbeSet9,const char *ProbeSet10, const char *ProbeSet11, const char *ProbeSet12)
{
	const char *libPath=".\\CDF";

	IGDACDATFilePtr pDatFile(CLSID_GDACDATFile);
	IGDACCDFFilePtr pCdfFile(CLSID_GDACCDFFile);
	IGDACProbeSetPtr pProbeSet(CLSID_GDACProbeSet);
	IGDACProbePairPtr pProbePair(CLSID_GDACProbePair);
	
	CTime CurTime = CTime::GetCurrentTime();
	int day=CurTime.GetDay();
	int month=CurTime.GetMonth();
	int minute=CurTime.GetMinute();
	int hour=CurTime.GetHour();
	int year=CurTime.GetYear();
	int sec=CurTime.GetSecond();

	//add time to file name
	CString dayC;
	dayC.Format("%d", day);
	CString monthC;
	monthC.Format("%d", month);
	CString minuteC;
	minuteC.Format("%d", minute);
	CString hourC;
	hourC.Format("%d", hour);
	CString yearC;
	yearC.Format("%d", year);
	CString secC;
	secC.Format("%d", sec);

	CString time=monthC+"-"+dayC+"-"+yearC+"-"+hourC+"h"+minuteC+"min"+secC+"sec";

    CString temp=datFile;
	temp.Replace(".DAT",time);
	// Create the output file
	
	CString outputFile;
	outputFile.Format("%s\\%s.TXT", dataPath, temp);
	ofstream outstr(outputFile, ios::out);
	
	// Output the column headers
	outstr <<
		"GeneChip" << "\t" <<
		"ProbeSet" << "\t\t\t" <<
		"Probe" << "\t\t\t" <<
		"ProbeNo" << "\t" <<
		"End" << "\t" <<
		"ProbeX" << "\t" <<
		"ProbeY" << "\t" <<
		"PM/MM" << "\t" <<
		"PixelX" << "\t" <<
		"PixelY" << "\t" <<
		"PixelID" << "\t\t" <<
		"Intensity" <<"\t"<< 
		"logpgc"  <<endl;

	// Read the DAT file.
	pDatFile->Name = datFile;
	pDatFile->DataPath = dataPath;
	pDatFile->LibPath = libPath;
	pDatFile->ServerName = "";
	if (pDatFile->Exists() == FALSE)
	{
		AfxMessageBox("Image file does not exist!");
		return "no";
	}
	pDatFile->Read();


	// Get the grid coordinates
	GRIDCOORDS grid;
	grid.upperleft.x = pDatFile->UpperLeftX;
	grid.upperleft.y = pDatFile->UpperLeftY;
	grid.upperright.x = pDatFile->UpperRightX;
	grid.upperright.y = pDatFile->UpperRightY;
	grid.lowerright.x = pDatFile->LowerRightX;
	grid.lowerright.y = pDatFile->LowerRightY;
	grid.lowerleft.x = pDatFile->LowerLeftX;
	grid.lowerleft.y = pDatFile->LowerLeftY;


	// Read the CDF file and load all the expression probe sets (ignore controls).
	pCdfFile->ChipType = pDatFile->ChipType;
	pCdfFile->LibPath = libPath;
	if (pCdfFile->Exists() == FALSE)
	{
		AfxMessageBox("CDF file does not exist!");
		return "no";
	}
	pCdfFile->Read();
	pCdfFile->FilterProbeSet = FILTER_ANALYSIS_PROBESET_TYPE;
	pCdfFile->LoadProbeSets();


	// Get the dimensions (in probes) of the array.
	int nChipSizeX = pCdfFile->Cols;
	int nChipSizeY = pCdfFile->Rows;


	// Loop over all probe sets then pairs
	CString strProbeSetName;
	int nNumLoaded = pCdfFile->NumProbeSetsLoaded;
	for (int iProbeSet=0; iProbeSet < nNumLoaded; iProbeSet++)
	{
		pCdfFile->GetProbeSet(iProbeSet, pProbeSet);
		strProbeSetName = (LPCTSTR) pProbeSet->Name;

		if(strcmp( strProbeSetName, ProbeSet1 ) == 0 || strcmp( strProbeSetName, ProbeSet2 ) == 0 || strcmp( strProbeSetName, ProbeSet3 ) == 0 
			|| strcmp( strProbeSetName, ProbeSet4 ) == 0 || strcmp( strProbeSetName, ProbeSet5 ) == 0 || strcmp( strProbeSetName, ProbeSet6 ) == 0
			||strcmp( strProbeSetName, ProbeSet7 ) == 0 || strcmp( strProbeSetName, ProbeSet8 ) == 0 || strcmp( strProbeSetName, ProbeSet9 ) == 0 
			|| strcmp( strProbeSetName, ProbeSet10 ) == 0 || strcmp( strProbeSetName, ProbeSet11 ) == 0 || strcmp( strProbeSetName, ProbeSet12) == 0)
		{
		// Loop over all pairs.
		int nNumPairs = pProbeSet->NumPairs;
		CRect PMCellRect;
				for (int iPair=0; iPair < nNumPairs; iPair++)
		{
			// For each pair get the pixel coordinates of the PM probe.
			pProbeSet->GetProbePair(iPair, pProbePair);

			CalcCellCorners((long*)&grid,
						pProbePair->PMPosX, pProbePair->PMPosY,
						nChipSizeX, nChipSizeY,
						(long*)&PMCellRect);

			OutputPixelData("PM", pProbePair->PMPosX, pProbePair->PMPosY, GeneChip,strProbeSetName, PMCellRect, pDatFile, outstr);
	
		}}
	}
	
	return temp;
}
